@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -155,7 +155,7 @@ Bitmap::Bitmap( const Bitmap & source, const Rectangle & r ) throw()
   }
 
 
-Bitmap::Bitmap( const Bitmap & source, int th, int scale ) throw()
+Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
   {
   if( scale < 2 || scale > source._width || scale > source._height ||
       th >= scale * scale )
@@ -167,58 +167,31 @@ Bitmap::Bitmap( const Bitmap & source, int th, int scale ) throw()
     for( int row = 0; row < source._height; ++row )
       for( int col = 0; col < source._width; ++col )
         if( source.data[row][col] ) ++dots;
-    th = (scale * scale * dots) / (source._width * source._height);
+    th = ( (scale * scale * dots) - 1 ) / (source._width * source._height);
     std::fprintf( stderr, "automatic threshold = %d\n", th );
     }
 
   _width = source._width / scale;
-  if( _width * scale < source._width ) ++_width;
   _height = source._height / scale;
-  if( _height * scale < source._height ) ++_height;
 
   data.resize( _height );
   for( int row = 0; row < _height; ++row )
     {
+    int srow = (row * scale) + scale;
     data[row].resize( _width );
-
     for( int col = 0; col < _width; ++col )
       {
+      int scol = (col * scale) + scale;
       int counter = 0;
-      for( int i = 0; i < scale; ++i )
-        if( (row * scale) + i < source._height )
-          for( int j = 0; j < scale; ++j )
-            if( (col * scale) + j < source._width )
-              if( source.data[(row * scale) + i][(col * scale) + j] )
-                ++counter;
-      data[row][col] = ( counter > th );
+      for( int i = srow - scale; i < srow; ++i )
+        for( int j = scol - scale; j < scol; ++j )
+          if( source.data[i][j] && ++counter > th ) goto L1;
+      L1: data[row][col] = ( counter > th );
       }
     }
   }
 
 
-void floodfill4( const Bitmap & source, Bitmap & dest, int row, int col ) throw()
-  {
-  std::stack< int, std::vector< int > > st;
-
-  if( source.data[row][col] == false && dest.data[row][col] == false )
-    { dest.data[row][col] = true; st.push(row); st.push(col); }
-
-  while( !st.empty() )
-    {
-    col = st.top(); st.pop();
-    row = st.top(); st.pop();
-    if( col > 0 && source.data[row][col-1] == false && dest.data[row][col-1] == false )
-      { dest.data[row][col-1] = true; st.push(row); st.push(col-1); }
-    if( col < dest._width-1 && source.data[row][col+1] == false && dest.data[row][col+1] == false )
-      { dest.data[row][col+1] = true; st.push(row); st.push(col+1); }
-    if( row > 0 && source.data[row-1][col] == false && dest.data[row-1][col] == false )
-      { dest.data[row-1][col] = true; st.push(row-1); st.push(col); }
-    if( row < dest._height-1 && source.data[row+1][col] == false && dest.data[row+1][col] == false )
-      { dest.data[row+1][col] = true; st.push(row+1); st.push(col); }
-    }
-  }
-
-
 Bitmap::Bitmap( const Bitmap & source, type t ) throw( Error )
   {
   _width = source._width; _height = source._height;
@@ -254,105 +227,149 @@ Bitmap::Bitmap( const Bitmap & source, type t ) throw( Error )
   }
 
 
+void Bitmap::floodfill4( const Bitmap & source, Bitmap & dest,
+                         int row, int col ) throw()
+  {
+  std::stack< int, std::vector< int > > st;
+
+  if( source.data[row][col] == false && dest.data[row][col] == false )
+    { dest.data[row][col] = true; st.push(row); st.push(col); }
+
+  while( !st.empty() )
+    {
+    col = st.top(); st.pop();
+    row = st.top(); st.pop();
+    if( col > 0 && !source.data[row][col-1] && !dest.data[row][col-1] )
+      { dest.data[row][col-1] = true; st.push(row); st.push(col-1); }
+    if( col < dest._width-1 && !source.data[row][col+1] && !dest.data[row][col+1] )
+      { dest.data[row][col+1] = true; st.push(row); st.push(col+1); }
+    if( row > 0 && !source.data[row-1][col] && !dest.data[row-1][col] )
+      { dest.data[row-1][col] = true; st.push(row-1); st.push(col); }
+    if( row < dest._height-1 && !source.data[row+1][col] && !dest.data[row+1][col] )
+      { dest.data[row+1][col] = true; st.push(row+1); st.push(col); }
+    }
+  }
+
+
 int Bitmap::analyse_layout( int layout_level ) throw()
   {
   if( _rectangle_vector.size() != 0 )
     Ocrad::internal_error( "analyse_layout called twice" );
 
-  Rectangle r( 0, 0, _width - 1, _height - 1 );
-  _rectangle_vector.push_back( r );
-  if( layout_level <= 0 ) return _rectangle_vector.size();
-
-  find_columns( 0 );
-  if( layout_level <= 1 ) return _rectangle_vector.size();
-
-  find_rows_and_columns();
+  if( layout_level > 0 && _width > 200 && _height > 200 )
+    {
+    Bitmap reduced( *this, 10, 9 );
+    Rectangle r( 0, 0, reduced._width - 1, reduced._height - 1 );
+    reduced._rectangle_vector.push_back( r );
+    reduced.find_columns( 0 );
+    if( layout_level >= 2 ) reduced.find_rows_and_columns();
+    if( reduced.rectangles() > 1 )
+      for( int i = 0; i < reduced.rectangles(); ++i )
+        {
+        const Rectangle & r = reduced._rectangle_vector[i];
+        Rectangle r1( 10*r.left(), 10*r.top(), 10*r.right(), 10*r.bottom() );
+        _rectangle_vector.push_back( r1 );
+        }
+    }
+  if( _rectangle_vector.size() == 0 )
+    {
+    Rectangle r( 0, 0, _width - 1, _height - 1 );
+    _rectangle_vector.push_back( r );
+    }
   return _rectangle_vector.size();
   }
 
 
-bool Bitmap::find_columns( int rindex ) throw()
+int Bitmap::find_columns( int rindex ) throw()
   {
-  bool split = false;
-//  for( int rindex = 0; rindex < rectangles(); )
+  const int colmin = ( _width > 100 ) ? _width / 10 : 10;
+  const int gapmin = ( _width > 200 ) ? _width / 100 : 2;
+  int columns = 0;
+  const Rectangle & r = _rectangle_vector[rindex];
+  std::vector< int > h_outline;
+  h_outline.resize( r.width() );
+
+  int total_dots = 0;
+  for( int col = 0; col < r.width(); ++col )
     {
-    int columns = 0;
-    const Rectangle & r = _rectangle_vector[rindex];
-    const int colmin = ( r.width() / 10 ) + 100;
-    const int gapmin = ( r.width() / 100 ) + 10;
-    std::vector< int > h_outline;
-    h_outline.resize( r.width() );
-
-    int total_dots = 0;
-    for( int col = 0; col < r.width(); ++col )
-      {
-      h_outline[col] = 0;
-      for( int row = 0; row < r.height(); ++row )
-        if( data[r.top()+row][r.left()+col] ) ++h_outline[col];
-      total_dots += h_outline[col];
-      }
-    int threshold_col = ( total_dots / ( r.width() * 4 ) ) + 6;
-    int threshold_gap = ( total_dots / ( r.width() * 20 ) ) + 1;
-
-    int left, right, col_width, gap_width;
-    for( left = 0; left < r.width(); ++left )	// cut left border
-      if( h_outline[left] >= threshold_gap ) break;
-    if( left < _width / 10 )			// cut left border noise
-      {
-      int left2 = left + gapmin;
-      for( ; left2 < r.width(); ++left2 )
-        if( h_outline[left2] >= threshold_gap ) break;
-      if( left2 > left + ( 2 * gapmin ) ) left = left2;
-      }
-    right = left + 1;
-    left = ( left >= gapmin ) ? left - gapmin : 0;
-
-    while( true )
-      {
-      const Rectangle & r = _rectangle_vector[rindex];
-      for( col_width = 0; right < r.width(); ++right )  // look for text
-        {
-        if( h_outline[right] > threshold_col ) ++col_width;
-        else if( col_width < colmin ) col_width = 0;
-        else break;
-        }
-      for( gap_width = 0; right < r.width(); ++right )  // look for gap
-        {
-        if( h_outline[right] < threshold_gap ) ++gap_width;
-        else if( gap_width < gapmin ) gap_width = 0;
-        else break;
-        }
-      if( right >= r.width() )
-        {
-        if( columns == 0 ) break;
-        ++columns;
-        Rectangle r1( r.left() + left, r.top(), r.right(), r.bottom() );
-        _rectangle_vector.insert( _rectangle_vector.begin() + rindex + columns, r1 );
-        break;
-        }
-      ++columns;
-      Rectangle r1( r.left() + left, r.top(), r.left() + right - ( gap_width / 2 ), r.bottom() );
-      _rectangle_vector.insert( _rectangle_vector.begin() + rindex + columns, r1 );
-      left = right - ( gap_width / 2 );
-      }
-    if( columns > 0 )
-      { _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
-      rindex += columns; split = true; }
-    else ++rindex;
+    h_outline[col] = 0;
+    for( int row = 0; row < r.height(); ++row )
+      if( data[r.top()+row][r.left()+col] ) ++h_outline[col];
+    total_dots += h_outline[col];
     }
-  return split;
+  if( 10 * total_dots > 8 * r.height() * r.width() )	// eliminates images
+    { _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
+    return true; }
+
+  int threshold_col = ( total_dots / ( r.width() * 4 ) ) + 6;
+  int threshold_gap = ( total_dots / ( r.width() * 20 ) ) + 1;
+
+  int left, right, right_end, col_width, gap_width;
+  for( left = 0; left < r.width(); ++left )	// cut left border
+    if( h_outline[left] >= threshold_gap ) break;
+  if( left < _width / 10 )			// cut left border noise
+    {
+    int left2 = left + gapmin;
+    for( ; left2 < r.width(); ++left2 )
+      if( h_outline[left2] >= threshold_gap ) break;
+    if( left2 > left + ( 2 * gapmin ) ) left = left2;
+    }
+  right = left + 1;
+  left = ( left > gapmin ) ? left - gapmin : 0;
+
+  for( right_end = r.width(); right_end > left; --right_end ) // cut right border
+    if( h_outline[right_end-1] >= threshold_gap ) break;
+  right_end = ( right_end + gapmin < r.width() ) ? right_end + gapmin : r.width();
+
+  while( true )
+    {
+    const Rectangle & r = _rectangle_vector[rindex];
+    for( col_width = 0; right < right_end; ++right )  // look for text
+      {
+      if( h_outline[right] > threshold_col ) ++col_width;
+      else if( col_width < colmin ) col_width = 0;
+      else break;
+      }
+    for( gap_width = 0; right < right_end; ++right )  // look for gap
+      {
+      if( h_outline[right] < threshold_gap ) ++gap_width;
+      else if( gap_width < gapmin ) gap_width = 0;
+      else break;
+      }
+    if( right >= right_end )
+      {
+      if( columns == 0 ) break;
+      ++columns;
+      Rectangle r1( r.left() + left, r.top(), r.right(), r.bottom() );
+      _rectangle_vector.insert( _rectangle_vector.begin() + rindex + columns, r1 );
+      break;
+      }
+    if( right >= right_end - colmin && columns == 0 ) break;
+    ++columns;
+    Rectangle r1( r.left() + left, r.top(), r.left() + right - ( gap_width / 2 ), r.bottom() );
+    _rectangle_vector.insert( _rectangle_vector.begin() + rindex + columns, r1 );
+    left = right - ( gap_width / 2 );
+    }
+  if( columns > 0 )
+    _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
+  return columns;
   }
 
 
 bool Bitmap::find_rows_and_columns() throw()
   {
+  const int rowmin = ( _height > 100 ) ? _height / 10 : 10;
+  const int gapmin = ( _height > 200 ) ? _height / 100 : 2;
   bool split = false;
   for( int rindex = 0; rindex < rectangles(); )
     {
     Rectangle & r = _rectangle_vector[rindex];
-    const int rowmin = ( _height > 1000 ) ? _height / 10 : 100;
-    const int gapmin = ( _height > 2000 ) ? _height / 100 : 20;
-    if( r.height() < ( 2 * rowmin ) + gapmin ) { ++rindex; continue; }
+    if( r.height() < ( 2 * rowmin ) + gapmin )
+      {
+      int c = find_columns( rindex );
+      if( c > 0 ) { rindex += c; split = true; } else ++rindex;
+      continue;
+      }
     std::vector< int > v_outline;
     v_outline.resize( r.height() );
 
@@ -364,6 +381,10 @@ bool Bitmap::find_rows_and_columns() throw()
         if( data[r.top()+row][r.left()+col] ) ++v_outline[row];
       total_dots += v_outline[row];
       }
+    if( 10 * total_dots > 8 * r.height() * r.width() )	// eliminates images
+      { _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
+      continue; }
+
 //    int threshold_row = ( total_dots / ( r.height() * 4 ) ) + 6;
     int threshold_gap = ( total_dots / ( r.height() * 20 ) ) + 1;
 
@@ -372,31 +393,34 @@ bool Bitmap::find_rows_and_columns() throw()
       if( v_outline[top] > threshold_gap ) break;
     for( bottom = r.height() - 1; bottom > top; --bottom ) // cut bottom border
       if( v_outline[bottom] > threshold_gap ) break;
-    if( bottom - top < 2 * rowmin ) { ++rindex; continue; }
+    if( bottom - top < 2 * rowmin )
+      {
+      int c = find_columns( rindex );
+      if( c > 0 ) { rindex += c; split = true; } else ++rindex;
+      continue;
+      }
 
-    int t, b, best_top = 0, best_bottom = 0, gaps = 0;
-    for( t = top + gapmin; t < bottom; ++t )  // look for top of gap
-      if( v_outline[t] < threshold_gap )
+    int t, b;
+    for( t = b = top + gapmin; t < bottom - gapmin; ++t )
+      if( v_outline[t] < threshold_gap )	// look for top of gap
         {
-        ++gaps;
-        for( b = t + 1; b < bottom - rowmin; ++b)  // look for bottom of gap
+        for( b = t + 1; b < bottom; ++b )	// look for bottom of gap
           if( v_outline[b] > threshold_gap ) break;
-        if( b - t > best_bottom - best_top )
-          { best_top = t; best_bottom = b; }
+        if( b - t >= gapmin ) break;
         t = b;
         }
-    if( gaps == 0 )	// eliminates images
-      _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
-    else if( best_bottom - best_top >= gapmin )
+    if( b - t >= gapmin )
       {
-      int mid = r.top() + ( ( best_top + best_bottom ) / 2 );
-      Rectangle r1( r.left(), mid + 1, r.right(), r.bottom() );
-      r.bottom( mid );
+      // cut by a minimum
+      for( int i = t; i <= b; ++i ) if( v_outline[i] < v_outline[t] ) t = i;
+      for( int i = b; i >= t; --i ) if( v_outline[i] < v_outline[b] ) b = i;
+      Rectangle r1( r.left(), r.top() + b, r.right(), r.bottom() );
+      r.bottom( r.top() + t );
       _rectangle_vector.insert( _rectangle_vector.begin() + rindex + 1, r1 );
-      find_columns( rindex + 1 );
-      find_columns( rindex );
+      if( find_columns( rindex ) == 0 ) ++rindex;
       split = true;
       }
+    else if( find_columns( rindex ) > 0 ) split = true;
     else ++rindex;
     }
   return split;
@@ -520,15 +544,18 @@ void Bitmap::save( FILE * f, char filetype ) const throw()
   if( filetype != '1' && filetype != '4' ) filetype = '4';
   std::fprintf( f, "P%c\n%d %d\n", filetype, _width, _height );
 
-  for( int row = 0; row < _height; ++row )
+  if( filetype == '1' )
     {
-    if( filetype == '1' )
+    for( int row = 0; row < _height; ++row )
       {
       for( int col = 0; col < _width; ++col )
         std::putc( data[row][col] ? '1' : '0', f );
       std::putc( '\n', f );
       }
-    else
+    }
+  else	// filetype == '4'
+    {
+    for( int row = 0; row < _height; ++row )
       {
       unsigned char byte = 0, mask = 0x80;
       for( int col = 0; col < _width; ++col )
@@ -537,8 +564,7 @@ void Bitmap::save( FILE * f, char filetype ) const throw()
         mask >>= 1;
         if( mask == 0 ) { std::putc( byte, f ); byte = 0; mask = 0x80; }
         }
-      if( mask != 0x80 ) std::putc( byte, f );  // incomplete byte at end of row
+      if( mask != 0x80 ) std::putc( byte, f ); // incomplete byte at end of row
       }
     }
-  std::fclose( f );
   }
