@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,10 @@ class Bitmap
   std::vector< std::vector< bool > > data;
   std::vector< Rectangle > _rectangle_vector;
 
-  bool find_columns( int rindex ) throw();
+  int  find_columns( int rindex ) throw();
   bool find_rows_and_columns() throw();
-
+  static void floodfill4( const Bitmap & source, Bitmap & dest,
+                          int row, int col ) throw();
 public:
   struct Error
     {
@@ -42,7 +43,7 @@ public:
   // Creates a Bitmap from another Bitmap
   Bitmap( const Bitmap & source, const Rectangle & r ) throw();
   // Creates a reduced bitmap
-  Bitmap( const Bitmap & source, int th, int scale ) throw();
+  Bitmap( const Bitmap & source, int scale, int th ) throw();
   // Creates a "special" type of Bitmap
   Bitmap( const Bitmap &, type ) throw( Error );
 
@@ -56,7 +57,4 @@ public:
   int analyse_layout( int layout_level ) throw();
   int horizontalify( bool verbose = false ) throw();
   void save( FILE *, char filetype = '4' ) const throw();
-
-  friend void floodfill4( const Bitmap & source, Bitmap & dest, int row,
-                                                 int col ) throw();
   };
