@@ -377,3 +377,46 @@ void Block::print( FILE * outfile, int sp ) const throw()
       p->print( outfile, sp + 8 );
     }
   }
+
+
+void Block::shift_block( const Block & b ) throw()
+  {
+  int i = blocks() - 1;
+  for( ; i >= 0; --i )
+    {
+    Block & bi = _block_vector[i];
+    if( b.top() > bi.top() ) break;
+    if( b.top() == bi.top() && b.left() >= bi.left() ) break;
+    }
+  _block_vector.insert( _block_vector.begin() + i + 1, b );
+  }
+
+
+void Block::hierarchize_blocks( std::vector< Block > & block_vector ) throw()
+  {
+  int blocks = block_vector.size();
+  for( int i1 = 0; i1 < blocks; ++i1 )
+    {
+    for( int i2 = i1 + 1; i2 < blocks; )
+      {
+      Block & b1 = block_vector[i1];
+      Block & b2 = block_vector[i2];
+      if( b2.top() >= b1.bottom() ) break;
+      if( b1.strictly_includes( b2 ) )
+        {
+        if( ( b1.id() > 0 && b2.id() < 0 ) || ( b1.id() < 0 && b2.id() > 0 ) )
+          b1.shift_block( b2 );
+        else
+          {
+          for( int i3 = 0; i3 < b1.blocks(); ++i3 )
+            {
+            Block & b3 = b1._block_vector[i3];
+            if( b3.strictly_includes( b2 ) ) b3.shift_block( b2 );
+            }
+          }
+        block_vector.erase( block_vector.begin() + i2 ); --blocks;
+        }
+      else ++i2;
+      }
+    }
+  }
