@@ -54,5 +54,6 @@ public:
 
   void print( FILE * outfile, int sp ) const throw();
 
-  friend void hierarchize_blocks( std::vector< Block > & block_vector ) throw();
+  void shift_block( const Block & b ) throw();
+  static void hierarchize_blocks( std::vector< Block > & block_vector ) throw();
   };
