@@ -27,7 +27,7 @@
 
 namespace {
 
-void delete_small_blocks( std::vector< Block > & block_vector ) throw()
+void ignore_small_blocks( std::vector< Block > & block_vector ) throw()
   {
   int to = 0, blocks = block_vector.size();
   for( int from = 0; from < blocks; ++from )
@@ -51,7 +51,7 @@ void delete_small_blocks( std::vector< Block > & block_vector ) throw()
   }
 
 
-void delete_white_blocks( std::vector< Block > & block_vector ) throw()
+void ignore_white_blocks( std::vector< Block > & block_vector ) throw()
   {
   int i = 0, blocks = block_vector.size();
   while( i < blocks )
@@ -66,16 +66,17 @@ void delete_white_blocks( std::vector< Block > & block_vector ) throw()
 void order_blocks( std::vector< Block > & block_vector ) throw()
   {
   int blocks = block_vector.size();
-  for( int i = 0; i < blocks; ++i )
+  for( int i = 0; i < blocks; )
     {
     int j = i, best = i;
     while( ++j < blocks && block_vector[j].top() <= block_vector[best].bottom() )
       if( block_vector[j].left() < block_vector[best].left() ) best = j;
-    if( best != i )
+    if( best == i ) ++i;
+    else
       {
       Block temp = block_vector[best];
-      block_vector.erase( block_vector.begin() + best );
-      block_vector.insert( block_vector.begin() + i, temp );
+      while( best > i ) { block_vector[best] = block_vector[best-1]; --best; }
+      block_vector[i] = temp;
       }
     }
   }
@@ -105,33 +106,30 @@ void remove_top_bottom_noise( std::vector< Block > & block_vector ) throw()
 } // end namespace
 
 
-void hierarchize_blocks( std::vector< Block > & block_vector ) throw()
+int Blockmap::generate_black_id() { static int id = 0; return ++id; }
+
+int Blockmap::generate_white_id() { static int id = 0; return --id; }
+
+
+void Blockmap::add_point_to_block( int row, int col, int id ) throw()
   {
-  int blocks = block_vector.size();
-  for( int i1 = 0; i1 < blocks; ++i1 )
-    {
-    for( int i2 = i1 + 1; i2 < blocks; )
+  static int ib = -1, iw = -1;
+  if( id > 0 && ib >= 0 && ib < blocks() && _block_vector[ib].id() == id )
+    { _block_vector[ib].add_point( row, col ); return; }
+  if( id < 0 && iw >= 0 && iw < blocks() && _block_vector[iw].id() == id )
+    { _block_vector[iw].add_point( row, col ); return; }
+  for( int c = 0, i = blocks() - 1; i >= 0 ; --i, ++c )
+    if( _block_vector[i].id() == id )
       {
-      Block & b1 = block_vector[i1];
-      Block & b2 = block_vector[i2];
-      if( b2.top() >= b1.bottom() ) break;
-      if( b1.strictly_includes( b2 ) )
-        {
-        if( ( b1.id() > 0 && b2.id() < 0 ) || ( b1.id() < 0 && b2.id() > 0 ) )
-          b1._block_vector.push_back( b2 );
-        else
-          {
-          for( int i3 = 0; i3 < b1.blocks(); ++i3 )
-            {
-            Block & b3 = b1._block_vector[i3];
-            if( b3.strictly_includes( b2 ) ) b3._block_vector.push_back( b2 );
-            }
-          }
-        block_vector.erase( block_vector.begin() + i2 ); --blocks;
-        }
-      else ++i2;
+      _block_vector[i].add_point( row, col );
+      if( id > 0 ) ib = i; else iw = i;
+      if( c < _width ) return;
+      Block temp = _block_vector[i];
+      _block_vector.erase( _block_vector.begin() + i );
+      _block_vector.push_back( temp );
+      return;
       }
-    }
+  Ocrad::internal_error( "add_point_to_block, lost block" );
   }
 
 
@@ -179,43 +177,23 @@ void Blockmap::join_blocks( int id1, int id2 ) throw()
   }
 
 
-void Blockmap::add_point_to_block( int row, int col, int id ) throw()
-  {
-  static int ib = -1, iw = -1;
-  if( id > 0 && ib >= 0 && ib < blocks() && _block_vector[ib].id() == id )
-    { _block_vector[ib].add_point( row, col ); return; }
-  if( id < 0 && iw >= 0 && iw < blocks() && _block_vector[iw].id() == id )
-    { _block_vector[iw].add_point( row, col ); return; }
-  for( int c = 0, i = blocks() - 1; i >= 0 ; --i, ++c )
-    if( _block_vector[i].id() == id )
-      {
-      _block_vector[i].add_point( row, col );
-      if( id > 0 ) ib = i; else iw = i;
-      if( c < _width ) return;
-      Block temp = _block_vector[i];
-      _block_vector.erase( _block_vector.begin() + i );
-      _block_vector.push_back( temp );
-      return;
-      }
-
-  Ocrad::internal_error( "add_point_to_block, lost block" );
-  }
-
-
-void Blockmap::delete_wide_blocks() throw()
+void Blockmap::ignore_wide_blocks() throw()
   {
   int i = 0, blocks = _block_vector.size();
   while( i < blocks )
     {
     const Block & b = _block_vector[i];
-    if( 2 * b.width() > _width )
+    if( 2 * b.width() < _width ) { ++i; continue; }
+    if( b.id() > 0 && 4 * b.area() > b.width() * b.height() )
       {
-      if( 10 * b.width() > 8 * _width && 10 * b.height() > 8 * _height &&
-          4 * b.area() > b.width() * b.height() )
+      // image, not frame
+      if( 10 * b.width() > 8 * _width && 10 * b.height() > 8 * _height )
         { _block_vector.clear(); return; }
-      _block_vector.erase( _block_vector.begin() + i ); --blocks;
+      for( int j = blocks - 1; j > i; --j )
+        if( b.includes( _block_vector[j] ) )
+          { _block_vector.erase( _block_vector.begin() + j ); --blocks; }
       }
-    else ++i;
+    _block_vector.erase( _block_vector.begin() + i ); --blocks;
     }
   }
 
@@ -295,10 +273,10 @@ Blockmap::Blockmap( const Bitmap & page_image, int rindex, int debug_level )
     }
   if( debug_level <= 99 )
     {
-    delete_small_blocks( _block_vector );
-    delete_wide_blocks();
-    hierarchize_blocks( _block_vector );
-    delete_white_blocks( _block_vector );
+    ignore_small_blocks( _block_vector );
+    ignore_wide_blocks();
+    Block::hierarchize_blocks( _block_vector );
+    ignore_white_blocks( _block_vector );
     remove_top_bottom_noise( _block_vector );
     if( debug_level <= 97 ) order_blocks( _block_vector );
     }
