@@ -26,15 +26,15 @@ class Blockmap
   std::vector< std::vector< int > > data;
   std::vector< Block > _block_vector;
 
-  int generate_black_id() { static int id = 0; return ++id; }
-  int generate_white_id() { static int id = 0; return --id; }
+  int generate_black_id();
+  int generate_white_id();
 
+  void add_point_to_block( int row, int col, int id ) throw();
   void create_block( const Rectangle & r, int id ) throw();
   void delete_block( int id ) throw();
   void join_blocks( int id1, int id2 ) throw();
-  void add_point_to_block( int row, int col, int id ) throw();
 
-  void delete_wide_blocks() throw();
+  void ignore_wide_blocks() throw();
 
 public:
   Blockmap( const Bitmap & source, int rindex, int debug_level ) throw();
