@@ -48,14 +48,12 @@ public:
   int area() const throw();
   const std::vector< Block > & block_vector() const throw()
     { return _block_vector; }
-//  const std::vector< unsigned char, int > & guess_vector() const throw()
-//    { return _guess; }
   const Blockmap * blockmap() const throw() { return _blockmap; }
+  int blocks() const throw() { return _block_vector.size(); }
 
-  void add_block( const Block & b ) throw();
   void only_block( const Block & b ) throw()
     { _block_vector.clear(); _block_vector.push_back( b ); }
-  int blocks() const throw() { return _block_vector.size(); }
+  void shift_block( const Block & b ) throw();
 
   void add_guess( unsigned char ch, int value ) throw();
   void insert_guess( int i, unsigned char ch, int value ) throw();
@@ -64,7 +62,6 @@ public:
   void swap_guesses( int i, int j ) throw();
   const Guess & guess( int i ) const throw();
   int guesses() const throw() { return _guess.size(); }
-  bool isalnum() const throw();
 
   void join( Character & c ) throw();
   void print( FILE * outfile, bool graph, bool recursive ) const throw();
