@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 
 void Ocrad::internal_error( const char * s ) throw()
   {
-  std::fprintf( stderr, "%s internal error: %s.\n", PROGRAM_NAME, s );
+  std::fprintf( stderr, "ocrad: internal error: %s.\n", s );
   exit( 3 );
   }
 
