@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -16,12 +16,6 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-// Date of this version: 2003-09-03
-
-#define PROGRAM_NAME    "ocrad"
-#define PROGRAM_VERSION "0.4"
-#define PROGRAM_YEAR    "2003"
-
 
 namespace Ocrad
   {
