@@ -55,7 +55,8 @@ int Features::hbars() const throw()
         {
         if( bm.id( row, col ) == b.id() )
           { ++c; x = col; if( col < b.right() ) continue; }
-        if( c > maxcount ) { maxcount = c; rt = x; lt = rt - c + 1; } c = 0;
+        if( c > maxcount ) { maxcount = c; rt = x; lt = rt - c + 1; }
+        c = 0;
         }
       switch( state )
         {
@@ -68,6 +69,7 @@ int Features::hbars() const throw()
                   if( rt > r ) r = rt;
                   if( row < b.bottom() ) break;
                   }
+                state = 0;
                 int end = ( maxcount <= limit ) ? row - 1 : row;
                 int width = r - l + 1;
                 while( begin <= end && 3 * count[begin-b.top()] < 2 * width )
@@ -76,7 +78,7 @@ int Features::hbars() const throw()
                   --end;
                 if( begin > end ) break;
                 _hbar.push_back( Rectangle( l, begin, r, end ) );
-                ++_hbars; state = 0; break;
+                ++_hbars; break;
         }
       }
     }
@@ -239,6 +241,9 @@ unsigned char Features::test_235Esz() const throw()
       {
       if( b.escape_bottom( urow3, ucol3 ) && b.escape_top( lrow1, lcol1 ) )
         return '*';
+      int m = lp.minima(); if( m < 2 || m > 3 ) return 0;
+      if( !bp.ispit() && m != 3 ) return 0;
+      if( b.height() > 3 * b.width() ) return 0;
       return '3';
       }
     if( b.escape_right( urow2, ucol2 ) )
@@ -301,7 +306,13 @@ unsigned char Features::test_49ARegpq() const throw()
     if( ( lp.ispit() && b.seek_bottom( row, h.right() ) < b.bottom() ) ||
         ( lp.isconvex() && b.seek_bottom( row, h.hcenter() ) < b.bottom() ) )
       return 'e';
-    else return 'p';
+    if( bp.ispit() )
+      {
+      int row2 = b.seek_bottom( row, h.right() );
+      if( row2 < b.vpos(75) ) return 'g';
+      if( row2 < b.bottom() ) return 'e';
+      }
+    return 'p';
     }
 
   if( b.escape_left( row, col ) )
@@ -501,11 +512,12 @@ unsigned char Features::test_CEFIJLlT() const throw()
         return 'L';
       }
 
-    if( vbar(0).left() > b.hcenter() )
+    if( vbar(0).left() > b.hcenter() &&
+        hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
+        hbar(0).width() + 1 >= b.width() )
       {
-      if( hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
-          hbar(0).width() + 1 >= b.width() )
-        return ISO_8859_1::NOT;
+      if( b.width() > b.height() ) return ISO_8859_1::NOT;
+      return 0;
       }
     }
 
@@ -604,7 +616,7 @@ unsigned char Features::test_frst() const throw()
     if( b.top_hook( &t_hdiff ) )
       {
       if( 3 * t_hdiff > 2 * b.height() && b.height() > 2 * wp.max() &&
-          tp.iminimum() > tp.pos( 50 ) && bp.iminimum() < bp.pos( 50 ) &&
+          tp.iminimum() > tp.pos( 50 ) && bp.iminimum() <= bp.pos( 50 ) &&
           ( !b_hdiff || rp.increasing( rp.pos( 50 ) ) ) )
         return 'f';
       if( 2 * b_hdiff > b.height() && 2 * t_hdiff > b.height() ) return 's';
@@ -614,32 +626,39 @@ unsigned char Features::test_frst() const throw()
   if( vbars() != 1 || vbar(0).width() < 2 ) return 0;
   if( vbar(0).hcenter() <= b.hcenter() )
     {
-    if( Ocrad::similar( b.height(), b.width(), 40 ) )
+    int col = b.right() - rp[b.vcenter()-b.top()] + 2;
+    if( col < b.right() )
       {
-      int col = b.right() - rp[b.vcenter()-b.top()] + 2;
       int row = b.seek_bottom( b.vcenter(), col );
       if( row >= b.bottom() || b.escape_bottom( row - 1, col ) )
         {
         if( rp.minima() == 3 ) return 'f';
-        if( tp.minima( b.height() / 8 ) == 2 &&
-            bp.minima( b.height() / 8 ) == 2 ) return 'x';
-        int row = b.vpos(75);
-        int col = b.seek_right( row, b.hcenter(), false ) + 1;
-        if( b.seek_right( row, col ) >= b.right() )
+        if( Ocrad::similar( b.height(), b.width(), 40 ) )
           {
-          if( lp.isconvex() ) return 'c';
-          if( hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
-              4 * hbar(0).height() <= b.height() &&
-              4 * lp[lp.pos(50)] >= b.width() ) return 'T';
-          return 'r';
+          if( tp.minima( b.height() / 8 ) == 2 &&
+              bp.minima( b.height() / 8 ) == 2 ) return 'x';
+          int row2 = b.vpos(75);
+          int col2 = b.seek_right( row2, b.hcenter(), false ) + 1;
+          if( b.seek_right( row2, col2 ) >= b.right() )
+            {
+            if( lp.isconvex() && ( col > b.hcenter() || row < b.bottom() ) )
+              return 0;
+            if( hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
+                4 * hbar(0).height() <= b.height() &&
+                4 * lp[lp.pos(50)] >= b.width() ) return 'T';
+            return 'r';
+            }
           }
         }
-      if( vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
-          b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
-        return 'x';
-      if( lp.isvpit() ) return 't';
+      if( Ocrad::similar( b.height(), b.width(), 40 ) )
+        {
+        if( vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
+            b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
+          return 'x';
+        if( lp.isvpit() ) return 't';
+        }
       }
-    else if( b.height() > b.width() && vbar(0).left() > b.left() &&
+    if( 3 * b.height() >= 5 * b.width() && vbar(0).left() > b.left() &&
              rp.minima() <= 2 )
       {
       int col = b.right() - rp[b.vcenter() - b.top()] + 1;
@@ -899,7 +918,7 @@ unsigned char Features::test_solid( int charbox_vcenter ) const throw()
     inner_area = 0;
     for( int row = b.top() + 1; row < b.bottom(); ++row )
       {
-      int holes = 0;
+      int holes = 0;	// FIXME
       for( int col = b.left() + 1; col < b.right(); ++col )
         { if( bm.id( row, col ) == b.id() ) ++inner_area; else ++holes; }
       if( 5 * holes >= b.width() ) return 0;
