@@ -138,6 +138,39 @@ bool ISO_8859_1::isalpha( unsigned char ch ) throw()
   }
 
 
+bool ISO_8859_1::islower( unsigned char ch ) throw()
+  {
+  if( std::islower( ch ) ) return true;
+  ch = base_letter( ch );
+  return ( ch && std::islower( ch ) );
+  }
+
+
+bool ISO_8859_1::islower_small( unsigned char ch ) throw()
+  {
+  if( !std::islower( ch ) ) return false;
+  switch( ch )
+    {
+    case 'a': case 'c': case 'e': case 'm': case 'n': case 'o':
+    case 'r': case 's': case 'u': case 'v': case 'w': case 'x':
+    case 'z': return true;
+    default : return false;
+    }
+  }
+
+
+bool ISO_8859_1::islower_small_ambiguous( unsigned char ch ) throw()
+  {
+  if( !std::islower( ch ) ) return false;
+  switch( ch )
+    {
+    case 'c': case 'o': case 's': case 'u': case 'v': case 'w': case 'x':
+    case 'z': return true;
+    default : return false;
+    }
+  }
+
+
 bool ISO_8859_1::isupper( unsigned char ch ) throw()
   {
   if( std::isupper( ch ) ) return true;
