@@ -102,6 +102,9 @@ public:
                                 unsigned char accent ) throw();
   static bool isalnum( unsigned char ch ) throw();
   static bool isalpha( unsigned char ch ) throw();
+  static bool islower( unsigned char ch ) throw();
+  static bool islower_small( unsigned char ch ) throw();
+  static bool islower_small_ambiguous( unsigned char ch ) throw();
   static bool isupper( unsigned char ch ) throw();
   static bool isvowel( unsigned char ch ) throw();
   static unsigned char toupper( unsigned char ch ) throw();
