@@ -55,5 +55,11 @@ public:
   bool v_includes( int row ) const throw();
   bool h_overlaps( const Rectangle & r ) const throw();
   bool v_overlaps( const Rectangle & r ) const throw();
-//  bool precedes( const Rectangle & r ) const throw();
+
+  int distance( const Rectangle & r ) const throw();
+  int distance( int row, int col ) const throw();
+  int h_distance( const Rectangle & r ) const throw();
+  int h_distance( int col ) const throw();
+  int v_distance( const Rectangle & r ) const throw();
+  int v_distance( int row ) const throw();
   };
