@@ -35,18 +35,22 @@ Textblock::Textblock( const std::vector< Block > & block_vector ) throw()
   for( std::vector< Block >::const_iterator p = block_vector.begin();
        p != block_vector.end(); ++p )
     {
-    if( p->height() * p->width() < 4 ) continue;
-    if( p->height() > 20 * p->width() ) continue;
-    if( p->width() > 20 * p->height() ) continue;
     Character c( *p );
-    if( prev && ( c.right() < prev->left() || c.top() > prev->bottom() ) )
+    if( c.height() * c.width() < 4 ) continue;
+    if( c.height() > 35 * c.width() ) continue;
+    if( c.width() > 20 * c.height() ) continue;
+    const Block * next = 0;
+    if( p + 1 != block_vector.end() ) next = &(*(p+1));
+    if( prev && ( c.right() < prev->left() || c.top() > prev->bottom() ) &&
+        ( !next || next->right() < prev->left() || next->top() > prev->bottom() ) )
       {
       int i;
       for( i = 0; i < line( current_line ).characters(); ++i )
         {
         Character & ci = line( current_line ).character(i);
         if( c.top() <= ci.bottom() &&
-            ( i > 0 || ci.height() <= 2 * c.height() ||
+            ( i > 0 || c.right() < ci.left() ||
+              ci.height() <= 2 * c.height() ||
               ci.height() <= 2 * line( current_line ).mean_height() ) ) break;
         }
       if( i >= line( current_line ).characters() ) prev = 0;
@@ -152,20 +156,14 @@ Textblock::Textblock( const std::vector< Block > & block_vector ) throw()
         for( int i = 1; i < newlines; ++i ) insert_line( ++current_line );
         }
     }
-  }
 
-
-int Textblock::alnum_percent() const throw()
-  {
-  int total = 0;
   for( int i = 0; i < lines(); ++i )
     {
-    Textline & line1 = line(i);
-    for( int j = 0; j < line1.characters(); ++j )
-      if( line1.character(j).isalnum() ) ++total;
+    // First pass. Recognize the easy characters.
+    line(i).recognize1();
+    // Second pass. Use context to clear up ambiguities.
+    line(i).recognize2();
     }
-  if( total == 0 ) return total;
-  return ( 100 * total ) / characters();
   }
 
 
