@@ -20,22 +20,17 @@ class Textblock
   {
   mutable std::vector< Textline > data;
 
-public:
-  Textblock( const std::vector< Block > & block_vector ) throw();
-
-  int alnum_percent() const throw();
   void add_line() throw() { data.push_back( Textline() ); }
   void insert_line( int ) throw();
   void delete_line( int ) throw();
+
+public:
+  Textblock( const std::vector< Block > & block_vector ) throw();
+
   Textline & line( int ) const throw();
   int lines() const throw() { return data.size(); }
   int characters() const throw();
 
   void print( FILE * outfile, bool graph, bool recursive ) const throw();
   void xprint( FILE * exportfile ) const throw();
-
-  void recognize1() const throw()
-    { for( int i = 0; i < lines(); ++i ) line( i ).recognize1(); }
-  void recognize2() const throw()
-    { for( int i = 0; i < lines(); ++i ) line( i ).recognize2(); }
   };
